# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"LossRouting": "01_loss.ipynb",
         "CombinedLoss": "01_loss.ipynb",
         "RoutedAccumMetric": "02_metric.ipynb",
         "route_to_metric": "02_metric.ipynb",
         "mtl_metrics": "02_metric.ipynb"}

modules = ["loss.py",
           "metric.py"]

doc_url = "https://bdsaglam.github.io/fastmtl/"

git_url = "https://github.com/bdsaglam/fastmtl/tree/master/"

def custom_doc_links(name): return None
